/*
 * xhtmlmenu.c
 * xhtmlite
 *
 * by Doug Wolfe
 * doug@wolfescience.com 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */  
 
#include <gtk/gtk.h>
#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)
#define TOOLTIP_TIMEOUT 500

#include "xhtmlmenu.h"
#include "xhtmlwidget.h"
#include "support.h"
  /* TODO (#1#): Add default menu for editing and formatting and 
                 inserting */
  
GtkWidget *textview_active;


static gint
cmp_families (const void *a, const void *b) 
{
	const char *a_name = pango_font_family_get_name (*(PangoFontFamily **) a);
	const char *b_name = pango_font_family_get_name (*(PangoFontFamily **) b);
	return g_utf8_collate (a_name, b_name);
}


static GdkPixbuf*
xhtmlite_file_encode_pixbuf_base64 (const gchar *filename)
{
	GError *error = NULL;

    // Open pixbuf
	GdkPixbuf *mypixbuf = gdk_pixbuf_new_from_file (filename, &error);

	if (error != NULL)
	{
		g_warning ("gdk_pixbuf_new_from_file, %s", error->message);
		return;
	}

	// read file
	gchar *pImageData;
	size_t ImageSize;
	gboolean file_completly_read =
			g_file_get_contents (filename, &pImageData, &ImageSize, &error);

	if (error != NULL)
	{
		g_warning ("g_file_get_contents, %s", error->message);
		return;
	}

	printf ("Image Size:%d\n", ImageSize);

	// encode pixbuf
	if (file_completly_read)
	{
	/*
		* if there is trouble with encoding the file
		try increasing
		* the size of BASE64_RESULT_SZ
		base64.c
	*/
		const gchar *encoded;
		encoded = base64_encode_bin(pImageData, ImageSize);

	// printf("Encoded Size:%d\n", g_str_get_length(encoded));
	// printf("filename:%s; imagesize:%d; file_encoded:%s; file
	// finished.\n", filename, ImageSize, encoded);
	
	// set file_string attached to pixbuf
		g_object_set_data (G_OBJECT (mypixbuf), "file_string", (gchar *)encoded);
    
	}
	// free data
	g_free (pImageData);
	return mypixbuf;
}


static 
GtkTextBuffer *xhtmlite_find_buffer ()
{
	if (textview_active == NULL)
	{
		printf ("**no active textview\n");
		return NULL;
	}
  
	GtkTextBuffer *textbuffer =
		gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview_active));
	printf ("buffer name=%s\n",
	   g_object_get_data (G_OBJECT (textbuffer), "name"));

	if (strcmp("xhtmlite_buffer",
		g_object_get_data (G_OBJECT (textbuffer), "name")) == 0)
	{
		return textbuffer;
	}
	else
	{
		return NULL;
	}
}


static void
on_toolbutton_cut_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL)
		return;
	gtk_text_buffer_cut_clipboard (buffer, gtk_clipboard_get (GDK_NONE), TRUE);
}


static void
on_toolbutton_copy_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL)
		return;
	gtk_text_buffer_copy_clipboard (buffer, gtk_clipboard_get (GDK_NONE));
}


static void
on_toolbutton_paste_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL)
		return;
	gtk_text_buffer_paste_clipboard (buffer,
			 gtk_clipboard_get (GDK_NONE), NULL, TRUE);
}


static void
on_toolbutton_delete_clicked (GtkToolButton *toolbutton,
					      gpointer user_data)
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL)
		return;
	gtk_text_buffer_delete_selection (buffer, TRUE, TRUE);
}


static void
on_toolbutton_clear_formatting_clicked (GtkToolButton *toolbutton,
		gpointer user_data)
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL)
		return;
	GtkTextIter begin, end;
	gtk_text_buffer_get_start_iter (buffer, &begin);
	gtk_text_buffer_get_end_iter (buffer, &end);
	gtk_text_buffer_remove_all_tags (buffer, &begin, &end);
}


static void
on_toolbutton_undo_clicked (GtkToolButton *toolbutton, gpointer user_data) 
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL)
    	return;
    
	xhtmlite_undo_undo(buffer);
	//xhtmlite_undo (buffer);
}


static void
on_toolbutton_redo_clicked (GtkToolButton *toolbutton, gpointer user_data) 
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL)
    	return;

	xhtmlite_undo_redo(buffer);
	//xhtmlite_redo (buffer);
}


static void
on_toggletoolbutton_bold_toggled (GtkToggleToolButton *toggletoolbutton,
				  gpointer user_data) 
{
  
GtkTextBuffer *buffer = xhtmlite_find_buffer ();
  
if (buffer == NULL)
    return;
  
xhtmlite_toggle_bold (buffer,
			 gtk_toggle_tool_button_get_active
			 (toggletoolbutton));

}


static void
on_toggletoolbutton_italic_toggled (GtkToggleToolButton *toggletoolbutton,
				    gpointer user_data) 
{
  
GtkTextBuffer *buffer = xhtmlite_find_buffer ();
  
if (buffer == NULL)
    return;
  
xhtmlite_toggle_italic (buffer,
			   gtk_toggle_tool_button_get_active
			   (toggletoolbutton));

}


static void
on_toggletoolbutton_underline_toggled (GtkToggleToolButton *toggletoolbutton, gpointer user_data) 
{
  
GtkTextBuffer *buffer = xhtmlite_find_buffer ();
  
if (buffer == NULL)
    return;
  
xhtmlite_toggle_underline (buffer,
			      gtk_toggle_tool_button_get_active
			      (toggletoolbutton));

}


static void
on_toggletoolbutton_strike_toggled (GtkToggleToolButton *toggletoolbutton,
				    gpointer user_data) 
{
  
GtkTextBuffer *buffer = xhtmlite_find_buffer ();
  
if (buffer == NULL)
    return;
  
xhtmlite_toggle_strikethrough (buffer, gtk_toggle_tool_button_get_active (toggletoolbutton));	// Strikethrough
}


static void
on_toggletoolbutton_superscript_toggled (GtkToggleToolButton *toggletoolbutton,
					 gpointer user_data) 
{
  
GtkTextBuffer *buffer = xhtmlite_find_buffer ();
  
if (buffer == NULL)
    return;
  
xhtmlite_toggle_superscript (buffer,
				gtk_toggle_tool_button_get_active
				(toggletoolbutton));

}


static void
on_toggletoolbutton_subscript_toggled (GtkToggleToolButton *toggletoolbutton, gpointer user_data) 
{
  
GtkTextBuffer *buffer = xhtmlite_find_buffer ();
  
if (buffer == NULL)
    return;
  
xhtmlite_toggle_subscript (buffer,
			      gtk_toggle_tool_button_get_active
			      (toggletoolbutton));

}


static void
on_toolbutton_symbol_clicked (GtkToolButton *toolbutton, gpointer user_data) 
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL) return;
  
	xhtmlite_open_symbol_dialog (xhtmlite_widget_find_parent_window(GTK_WIDGET(toolbutton)), buffer);
}

static void
on_toolbutton_insert_link_clicked (GtkToolButton *toolbutton, gpointer user_data) 
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL) return;

	xhtmlite_insert_link (xhtmlite_widget_find_parent_window(GTK_WIDGET(toolbutton)), buffer);
}

static GtkWidget *xhtmlite_dialog_import_image;

static void
on_toolbutton_inline_image_clicked (GtkToolButton *toolbutton,
				    gpointer user_data) 
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL) return;
  
	if (xhtmlite_dialog_import_image == NULL)
    {
		GtkWidget *parent_window = xhtmlite_widget_find_parent_window(GTK_WIDGET(toolbutton));
		xhtmlite_dialog_import_image = gtk_file_chooser_dialog_new ("Choose Image", GTK_WINDOW (parent_window),				     
				GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
      
		GtkFileFilter *filefilter_image = gtk_file_filter_new ();
		gtk_file_filter_set_name (filefilter_image,
				 "Image Files (*gif, *.jpg, *.png)");
		gtk_file_filter_add_pattern (filefilter_image, "*.gif");
		gtk_file_filter_add_pattern (filefilter_image, "*.jpg");
		gtk_file_filter_add_pattern (filefilter_image, "*.png");
		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER
				    (xhtmlite_dialog_import_image), filefilter_image);
      
		GtkFileFilter *filefilter_all = gtk_file_filter_new ();
		gtk_file_filter_set_name (filefilter_all, "All Files (*.*)");
		gtk_file_filter_add_pattern (filefilter_all, "*");
		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER
				    (xhtmlite_dialog_import_image), filefilter_all);
	}
  
	if (gtk_dialog_run (GTK_DIALOG (xhtmlite_dialog_import_image)) == GTK_RESPONSE_OK)
	{
		gtk_widget_hide (xhtmlite_dialog_import_image);

		gchar *filename;
		filename = 	gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(xhtmlite_dialog_import_image));
      
		GdkPixbuf *mypixbuf = (GdkPixbuf *) xhtmlite_file_encode_pixbuf_base64 (filename);
		
		// Insert pixbuf
		GtkTextIter beginselect, endselect;
		gtk_text_buffer_get_selection_bounds (buffer, &beginselect, &endselect);
		gtk_text_buffer_insert_pixbuf (buffer, &beginselect, mypixbuf);
      	
		// free data
		g_free (filename);
	}

	gtk_widget_hide (xhtmlite_dialog_import_image);
}


static void
on_combobox_font_changed (GtkComboBox *combobox, gpointer user_data)
{
	printf ("font changed");
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL)
		return;

	gchar *active_text = gtk_combo_box_get_active_text (combobox);
	xhtmlite_change_fontfamily (buffer, textview_active, active_text);
}


static void
on_toolbutton_fontcolor_clicked (GtkToolButton *toolbutton, gpointer user_data)
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL)
		return;
	xhtmlite_open_font_color_dialog (xhtmlite_widget_find_parent_window(GTK_WIDGET(toolbutton)), buffer);
}


static void
on_toolbutton_highlightcolor_clicked (GtkToolButton *toolbutton, gpointer user_data) 
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
  
	if (buffer == NULL)
    	return;
	xhtmlite_open_highlight_color_dialog (xhtmlite_widget_find_parent_window(GTK_WIDGET(toolbutton)), buffer);
}


static void
on_combobox_fontsize_absolute_changed (GtkComboBox *combobox,
				       gpointer user_data) 
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();

	if (buffer == NULL)
    	return;
	gchar *active_text = gtk_combo_box_get_active_text (combobox);
	xhtmlite_adjust_fontsize_absolute (buffer, active_text);
}


static void 
xhtmlite_track_tags (GtkWidget *textview) 
{
	GtkTextBuffer *buffer = xhtmlite_find_buffer ();
	if (buffer == NULL) return;
  
    // get tags
    GtkTextIter start, end;
	gtk_text_buffer_get_selection_bounds (buffer, &start, &end);
	gtk_text_iter_backward_char (&start);
	GSList *tags = gtk_text_iter_get_tags (&start);
  
    // get widgets
    GtkWidget *toggletoolbutton_bold = lookup_widget (textview, "toggletoolbutton_bold");
	GtkWidget *toggletoolbutton_italic = lookup_widget (textview, "toggletoolbutton_italic");
	GtkWidget *toggletoolbutton_underline = lookup_widget (textview, "toggletoolbutton_underline");
	GtkWidget *toggletoolbutton_strike = lookup_widget (textview, "toggletoolbutton_strike");
	GtkWidget *toggletoolbutton_superscript =lookup_widget (textview, "toggletoolbutton_superscript");
	GtkWidget *toggletoolbutton_subscript = lookup_widget (textview, "toggletoolbutton_subscript");
	GtkWidget *combobox_font = lookup_widget (textview, "combobox_font");
 	GtkWidget *combobox_fontsize_absolute = lookup_widget (textview, "combobox_fontsize_absolute");
	GtkWidget *combobox_fontsize_pt = lookup_widget (textview, "combobox_fontsize_pt");

  
    // block toggle handlers
    g_signal_handlers_block_matched (GTK_OBJECT
				     (toggletoolbutton_bold),
				     G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  
	g_signal_handlers_block_matched (GTK_OBJECT
				    (toggletoolbutton_italic),
				    G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  
	g_signal_handlers_block_matched (GTK_OBJECT
				    (toggletoolbutton_underline),
				    G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  
	g_signal_handlers_block_matched (GTK_OBJECT
				    (toggletoolbutton_strike),
				    G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  
	g_signal_handlers_block_matched (GTK_OBJECT
				    (toggletoolbutton_superscript),
				    G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  
	g_signal_handlers_block_matched (GTK_OBJECT
				    (toggletoolbutton_subscript),
				    G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  

	g_signal_handlers_block_matched (GTK_OBJECT (combobox_font),
				     G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
				     G_CALLBACK
				     (on_combobox_font_changed), NULL);
  
	g_signal_handlers_block_matched (GTK_OBJECT
				    (combobox_fontsize_absolute),
				    G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
				    G_CALLBACK
				    (on_combobox_fontsize_absolute_changed),
				    NULL);
  
    // TODO: Fix the below line, when you add font_pointsize change
	g_signal_handlers_block_matched (GTK_OBJECT
				     (combobox_fontsize_pt),
				     G_SIGNAL_MATCH_FUNC, 0, 0,
				     NULL,
				     G_CALLBACK
				     (on_combobox_font_changed), NULL);
  

	// set toggle button active to FALSE
	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON
				       (toggletoolbutton_bold), FALSE);
  
	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON
				      (toggletoolbutton_italic), FALSE);
  
	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON
				      (toggletoolbutton_underline), FALSE);
  
	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON
				      (toggletoolbutton_strike), FALSE);
  
	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON
				      (toggletoolbutton_superscript), FALSE);
  
	gtk_toggle_tool_button_set_active (GTK_TOGGLE_TOOL_BUTTON
				      (toggletoolbutton_subscript), FALSE);
  

    // get default font
    PangoFontDescription * default_font_desc = gtk_widget_get_style (GTK_WIDGET (textview))->font_desc;
  
	const gchar *default_fontfamily = pango_font_description_get_family (default_font_desc);
	gint default_fontsize = pango_font_description_get_size (default_font_desc);
  
    // set default fontfamily
    GtkTreeIter iter;
    gchar *iter_string;
    GtkTreeModel *model_fontfamily = gtk_combo_box_get_model (GTK_COMBO_BOX (combobox_font));
	gboolean has_iter = gtk_tree_model_get_iter_first (model_fontfamily, &iter);
	while (has_iter)
	{
		gtk_tree_model_get (model_fontfamily, &iter, 0, &iter_string, -1);
		if (g_ascii_strcasecmp (default_fontfamily, iter_string) == 0)
		{
			gtk_combo_box_set_active_iter (GTK_COMBO_BOX(combobox_font), &iter);
			break;
		}
		has_iter = gtk_tree_model_iter_next (model_fontfamily, &iter);
	}
  
    // set default fontfamily
    gtk_combo_box_set_active (GTK_COMBO_BOX (combobox_fontsize_absolute), 0);


	// Check tags
	int n;
	guint len = g_slist_length (tags);
	for (n = 0; n < len; n++)
	{
		GtkTextTag *tag = g_slist_nth_data (tags, n);
		if (strcmp (tag->name, "b") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON (toggletoolbutton_bold), TRUE);
		}
		else if (strcmp (tag->name, "i") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON (toggletoolbutton_italic), TRUE);
		}
		else if (strcmp (tag->name, "u") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON (toggletoolbutton_underline), TRUE);
		}
		else if (strcmp (tag->name, "del") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON (toggletoolbutton_strike), TRUE);
		}
		else if (strcmp (tag->name, "sup") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON (toggletoolbutton_superscript), TRUE);
		}
		else if (strcmp (tag->name, "sub") == 0)
		{
			gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON (toggletoolbutton_subscript), TRUE);
		}
		else if (g_strrstr (tag->name, "face") != NULL)
		{
			gchar *fontfamily;
			g_object_get (tag, "family", &fontfamily, NULL);
	  		// set selected font
	  		has_iter = gtk_tree_model_get_iter_first (model_fontfamily, &iter);

			while (has_iter)
			{
				gtk_tree_model_get (model_fontfamily, &iter,
					0, &iter_string, -1);
	      		if (strcmp (fontfamily, iter_string) == 0)
				{
					gtk_combo_box_set_active_iter
		    			(GTK_COMBO_BOX (combobox_font), &iter);
		  			break;
				}
	      		has_iter = gtk_tree_model_iter_next (model_fontfamily, &iter);
	    	}
		}
      	else if (g_strrstr (tag->name, "font-size") != NULL)
		{
			gchar *fontsize = g_object_get_data (G_OBJECT (tag), "font-size_absolute");
	  		printf ("fontsize:%s\n", fontsize);
	  
	  		// set selected font
			GtkTreeModel *model_fontsize_absolute = gtk_combo_box_get_model (GTK_COMBO_BOX (combobox_fontsize_absolute));
	  		has_iter = gtk_tree_model_get_iter_first (model_fontsize_absolute, &iter);
	  		while (has_iter)
	        {
	    		gtk_tree_model_get (model_fontsize_absolute, &iter, 0, &iter_string, -1);
	      		if (strcmp (fontsize, iter_string) == 0)
				{
		  			gtk_combo_box_set_active_iter(GTK_COMBO_BOX (combobox_fontsize_absolute), &iter);
		  			break;
				}
	      		has_iter = gtk_tree_model_iter_next (model_fontsize_absolute, &iter);
		    }
		}
	}
  
    g_signal_handlers_unblock_matched (GTK_OBJECT
				       (toggletoolbutton_bold),
				       G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  
	g_signal_handlers_unblock_matched (GTK_OBJECT
				      (toggletoolbutton_italic),
				      G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  
	g_signal_handlers_unblock_matched (GTK_OBJECT
				      (toggletoolbutton_underline),
				      G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  
	g_signal_handlers_unblock_matched (GTK_OBJECT
				      (toggletoolbutton_strike),
				      G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  
	g_signal_handlers_unblock_matched (GTK_OBJECT
				      (toggletoolbutton_superscript),
				      G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  
	g_signal_handlers_unblock_matched (GTK_OBJECT
				      (toggletoolbutton_subscript),
				      G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, NULL);
  
	g_signal_handlers_unblock_matched (GTK_OBJECT (combobox_font),
				       G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
				       G_CALLBACK
				       (on_combobox_font_changed), NULL);
  
	g_signal_handlers_unblock_matched (GTK_OBJECT
				      (combobox_fontsize_absolute),
				      G_SIGNAL_MATCH_FUNC, 0, 0, NULL,
				      G_CALLBACK
				      (on_combobox_fontsize_absolute_changed),
				      NULL);
  
    // TODO: Fix the below line, when you add font_pointsize chenage
    g_signal_handlers_unblock_matched (GTK_OBJECT
				       (combobox_fontsize_pt),
				       G_SIGNAL_MATCH_FUNC, 0, 0,
				       NULL,
				       G_CALLBACK
				       (on_combobox_font_changed), NULL);
}


static gboolean 
on_textview_key_release_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	textview_active = widget;
	xhtmlite_track_tags (textview_active);
	return FALSE;
}


static gboolean 
on_textview_button_release_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	textview_active = widget;
	xhtmlite_track_tags (textview_active);

	return FALSE;
}


static gboolean 
on_textview_focus_in_event (GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
	textview_active = widget;
	return FALSE;
}



static gboolean 
on_textview_focus_out_event (GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
{
	if(!gtk_widget_is_focus(widget))
	{
		textview_active = NULL;
	}
	return FALSE;
}

//static GtkWidget *xhtmlite_window_www_display = NULL;
static gint tip_timer = 0;
static GtkWidget *tip_window = NULL;
static gchar *tip = NULL;
static gint 
xhtmlite_tip_paint (GtkWidget *textview)
{
	/*code from edited from GAIM*/
	PangoLayout *layout;
	layout = gtk_widget_create_pango_layout(tip_window, tip);

	gtk_paint_flat_box (tip_window->style, tip_window->window,
						GTK_STATE_NORMAL, GTK_SHADOW_OUT, NULL, tip_window,
						"tooltip", 0, 0, -1, -1);

	gtk_paint_layout (tip_window->style, tip_window->window, GTK_STATE_NORMAL,
					  FALSE, NULL, tip_window, NULL, 4, 4, layout);

	g_object_unref(layout);
	return FALSE;
}
/*TODO: Fix this link tip window, so the window does not move with the cursor, \ 
		but stays visible.  Also, add a menu on the right-click of the link to \
		follow the link or copy the location. */

static gint
xhtmlite_tip (gpointer data)
{
	/*code from edited from GAIM*/
	PangoFontMetrics *font_metrics;
	PangoLayout *layout;
	PangoFont *font;
	GtkWidget *textview = data;
	
	gint gap, x, y, h, w, scr_w, baseline_skip;

	if (tip == NULL) 
	{
		tip_timer = 0;
		return FALSE;
	}

	if (tip_window)
	{
			gtk_widget_destroy (tip_window);
			tip_window = NULL;
	}

	tip_timer = 0;
	tip_window = gtk_window_new (GTK_WINDOW_POPUP);
	gtk_widget_set_app_paintable (tip_window, TRUE);
	gtk_window_set_resizable (GTK_WINDOW (tip_window), FALSE);
	gtk_widget_set_name (tip_window, "gtk-tooltips");
	g_signal_connect_swapped (G_OBJECT (tip_window), "expose_event",
							  G_CALLBACK (xhtmlite_tip_paint), textview);

	gtk_widget_ensure_style (tip_window);
	layout = gtk_widget_create_pango_layout(tip_window, tip);
	font = pango_context_load_font(pango_layout_get_context(layout),
			      tip_window->style->font_desc);

	if (font == NULL) {
		char *tmp = pango_font_description_to_string(
					tip_window->style->font_desc);

		//gaim_debug(GAIM_DEBUG_ERROR, "gtk_imhtml_tip",
		//	"pango_context_load_font() couldn't load font: '%s'\n",
		//	tmp);
		g_free(tmp);

		return FALSE;
		
}

	font_metrics = pango_font_get_metrics(font, NULL);
	

	pango_layout_get_pixel_size(layout, &scr_w, NULL);
	gap = PANGO_PIXELS((pango_font_metrics_get_ascent(font_metrics) +
					   pango_font_metrics_get_descent(font_metrics))/ 4);

	if (gap < 2)
		gap = 2;
	baseline_skip = PANGO_PIXELS(pango_font_metrics_get_ascent(font_metrics) +
								pango_font_metrics_get_descent(font_metrics));
	w = 8 + scr_w;
	h = 8 + baseline_skip;

	gdk_window_get_pointer (NULL, &x, &y, NULL);
	if (GTK_WIDGET_NO_WINDOW (textview))
		y += GTK_WIDGET(textview)->allocation.y;

	scr_w = gdk_screen_width();

	x -= ((w >> 1) + 4);

	if ((x + w) > scr_w)
		x -= (x + w) - scr_w;
	else if (x < 0)
		x = 0;

	y = y + PANGO_PIXELS(pango_font_metrics_get_ascent(font_metrics) +
						pango_font_metrics_get_descent(font_metrics));

	gtk_widget_set_size_request (tip_window, w, h);
	gtk_widget_show (tip_window);
	gtk_window_move (GTK_WINDOW(tip_window), x, y);

	pango_font_metrics_unref(font_metrics);
	g_object_unref(layout);

	return FALSE;
}

static gboolean 
on_textview_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, GtkStatusbar *statusbar)
{

	//GdkWindow *window = gtk_text_view_get_window (textview, GTK_TEXT_WINDOW_TEXT);
	/*first get the current iter*/
	gint x = 0;
	gint y = 0;
	GtkTextView * textview = GTK_TEXT_VIEW(widget);
	GdkModifierType state = 0;
	GdkWindow *window;

	if(event->is_hint)
	{
		window = gdk_window_get_pointer(event->window, &x, &y,&state);
	}
	else
	{
		x = event->x;
	    y = event->y;
	    state = event->state;
	}

	GtkTextIter iter;
	gtk_text_view_get_iter_at_location(textview, &iter, x, y);

	/*if the iter has a link tag, set the cursor*/
	GdkCursor *cursor = NULL;
	GSList *tags = gtk_text_iter_get_tags(&iter);
	GtkTextTag *tag = NULL;
	int n;
	guint len = g_slist_length(tags);
	GdkColor previous_color;
	for(n = 0; n < len; n++)
	{
		tag = g_slist_nth_data (tags, n);
		gchar *link = g_object_get_data(G_OBJECT(tag), "link");
		if(g_str_get_length(link) > 0)
		{
			cursor = gdk_cursor_new (GDK_HAND1);
			/*popup window*/
			tip = g_strdup(link);
			tip_timer = g_timeout_add (TOOLTIP_TIMEOUT, xhtmlite_tip, textview);
			/*statusbar*/
			if (statusbar != NULL) gtk_statusbar_push (GTK_STATUSBAR(statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "link"), link);
			
		}
	}
	
	if(cursor == NULL)
	{
		cursor = gdk_cursor_new (GDK_XTERM);
		/*popup window*/
		if (tip_window)
		{
			gtk_widget_destroy (tip_window);
			tip = NULL;
			tip_window = NULL;
		}
		/*statusbar*/
		if (statusbar != NULL) gtk_statusbar_pop (GTK_STATUSBAR(statusbar), gtk_statusbar_get_context_id(GTK_STATUSBAR(statusbar), "link"));
	}	
	gdk_window_set_cursor (event->window, cursor);
    gdk_flush ();
    gdk_cursor_destroy (cursor);

	return FALSE;
}

void 
xhtmlite_attach_textview_events (GtkWidget *textview, GtkWidget *statusbar) 
{
	g_signal_connect ((gpointer) textview, "key_release_event",
		G_CALLBACK (on_textview_key_release_event), NULL);
  
	g_signal_connect ((gpointer) textview, "button_release_event",		     
		G_CALLBACK (on_textview_button_release_event), NULL);

	g_signal_connect ((gpointer) textview, "focus_in_event", 	     
		G_CALLBACK (on_textview_focus_in_event), NULL);
  
	g_signal_connect ((gpointer) textview, "focus_out_event", 		     
		G_CALLBACK (on_textview_focus_out_event), NULL);
	
	g_signal_connect ((gpointer) textview, "motion_notify_event", 		     
		G_CALLBACK (on_textview_motion_notify_event), statusbar);
} 

GtkWidget* 
xhtmlite_create_default_toolbar_edit (GtkWidget *parent_window) 
{
	GtkWidget *handlebox;
	GtkWidget *toolbar;
	GtkWidget *toolitem;

	handlebox = gtk_handle_box_new ();

	gtk_widget_show (handlebox);
  
    // gtk_box_pack_start (GTK_BOX (dialog_byoj_vbox), handlebox, FALSE,
    // TRUE, 0);
    
	toolbar = gtk_toolbar_new ();

	gtk_widget_show (toolbar);
	gtk_container_add (GTK_CONTAINER (handlebox), toolbar);
	gtk_widget_set_size_request (toolbar, 500, -1);
	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
  
    // gtk_container_set_border_width (GTK_CONTAINER (toolbar), 2);
    // tmp_toolbar_icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR
    // (toolbar));
    
    /* Store pointers to all widgets, for use by lookup_widget(). */ 
    GLADE_HOOKUP_OBJECT (parent_window, toolbar, "toolbar_edit");

	toolitem = xhtmlite_toolbutton_cut ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toolbutton_cut");

	toolitem = xhtmlite_toolbutton_copy ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toolbutton_copy");

	toolitem = xhtmlite_toolbutton_paste ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toolbutton_paste");

	toolitem = xhtmlite_toolbutton_delete ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toolbutton_delete");
	
	toolitem = xhtmlite_toolbutton_clear_formatting ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"toolbutton_clear_formatting");

	toolitem = (GtkWidget *) gtk_separator_tool_item_new ();
	gtk_widget_show (toolitem);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"separatortoolitem1");

	toolitem = xhtmlite_toolbutton_undo ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toolbutton_undo");

	toolitem = xhtmlite_toolbutton_redo ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toolbutton_redo");

	toolitem = (GtkWidget *) gtk_separator_tool_item_new ();
	gtk_widget_show (toolitem);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"separatortoolitem2");

	toolitem = xhtmlite_toggletoolbutton_bold ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toggletoolbutton_bold");

	toolitem = xhtmlite_toggletoolbutton_italic ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toggletoolbutton_italic");

	toolitem = xhtmlite_toggletoolbutton_underline ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"toggletoolbutton_underline");

	toolitem = xhtmlite_toggletoolbutton_strike ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toggletoolbutton_strike");

	toolitem = (GtkWidget *) gtk_separator_tool_item_new ();
	gtk_widget_show (toolitem);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"separatortoolitem3");

	toolitem = xhtmlite_toggletoolbutton_superscript ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"toggletoolbutton_superscript");

	toolitem = xhtmlite_toggletoolbutton_subscript ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"toggletoolbutton_subscript");
  
	return handlebox;
}



GtkWidget* 
xhtmlite_create_default_toolbar_font (GtkWidget *parent_window) 
{
	GtkWidget *handlebox;
	GtkWidget *toolbar;
	GtkWidget *toolitem;
  
	handlebox = gtk_handle_box_new ();
	gtk_widget_show (handlebox);
  
    // gtk_box_pack_start (GTK_BOX (dialog_byoj_vbox), handlebox, FALSE,
    // TRUE, 0);
    
	toolbar = gtk_toolbar_new ();
	gtk_widget_show (toolbar);
	gtk_container_add (GTK_CONTAINER (handlebox), toolbar);
	gtk_widget_set_size_request (toolbar, 500, -1);

	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
  
    // gtk_container_set_border_width (GTK_CONTAINER (toolbar), 2);
    // tmp_toolbar_icon_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR
    // (toolbar));
    
    /*
     * Store pointers to all widgets, for use by lookup_widget(). 
     */ 
    GLADE_HOOKUP_OBJECT (parent_window, toolbar, "toolbar_edit");
  

	toolitem = xhtmlite_combobox_font (parent_window);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toolitem_combobox_font");

	toolitem = xhtmlite_combobox_fontsize_absolute (parent_window);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"toolitem_combobox_fontsize_absolute");

	toolitem = xhtmlite_combobox_fontsize_pt (parent_window);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"toolitem_combobox_fontsize_pt");

	toolitem = (GtkWidget *) gtk_separator_tool_item_new ();
	gtk_widget_show (toolitem);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"separatortoolitem4");

	toolitem = xhtmlite_toolbutton_fontcolor ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toolbutton_fontcolor");
  
	toolitem = xhtmlite_toolbutton_highlightcolor ();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"toolbutton_highlightcolor");
  
  	toolitem = (GtkWidget *) gtk_separator_tool_item_new ();
	gtk_widget_show (toolitem);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem,
			"separatortoolitem5");

	toolitem = xhtmlite_toolbutton_insert_link (parent_window);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toolbutton_insert_link");
  
	toolitem = xhtmlite_toolbutton_symbol (parent_window);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toolbutton_symbol");
  
	toolitem = xhtmlite_toolbutton_inline_image (parent_window);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar),GTK_TOOL_ITEM(toolitem),-1);
	GLADE_HOOKUP_OBJECT (parent_window, toolitem, "toolbutton_inline_image");
	return handlebox;
}



GtkWidget* 
xhtmlite_toolbutton_cut () 
{
  
GtkWidget *toolbutton =
    (GtkWidget *) gtk_tool_button_new_from_stock ("gtk-cut");
  
gtk_widget_show (toolbutton);
  
GtkTooltips *tooltips = gtk_tooltips_new ();
  
gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
			      _("Cut"), NULL);
  

g_signal_connect ((gpointer) toolbutton, "clicked",
		      
G_CALLBACK (on_toolbutton_cut_clicked), 
NULL);
  
return toolbutton;

}



GtkWidget* 
xhtmlite_toolbutton_copy () 
{
  
GtkWidget *toolbutton =
    (GtkWidget *) gtk_tool_button_new_from_stock ("gtk-copy");
  
gtk_widget_show (toolbutton);
  
GtkTooltips *tooltips = gtk_tooltips_new ();
  
gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
			      _("Copy"), NULL);
  

g_signal_connect ((gpointer) toolbutton, "clicked",
		      
G_CALLBACK (on_toolbutton_copy_clicked), 
NULL);
  
return toolbutton;

}



GtkWidget* 
xhtmlite_toolbutton_paste () 
{
  
GtkWidget *toolbutton =
    (GtkWidget *) gtk_tool_button_new_from_stock ("gtk-paste");
  
gtk_widget_show (toolbutton);
  
GtkTooltips *tooltips = gtk_tooltips_new ();
  
gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
			      _("Paste"), NULL);
  

g_signal_connect ((gpointer) toolbutton, "clicked",
		      
G_CALLBACK (on_toolbutton_paste_clicked), 
NULL);
  
return toolbutton;

}



GtkWidget* 
xhtmlite_toolbutton_delete () 
{
  
GtkWidget *toolbutton =
    (GtkWidget *) gtk_tool_button_new_from_stock ("gtk-delete");
  
gtk_widget_show (toolbutton);
  
GtkTooltips *tooltips = gtk_tooltips_new ();
  
gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
			      _("Delete"), NULL);
  

g_signal_connect ((gpointer) toolbutton, "clicked",
		      
G_CALLBACK (on_toolbutton_delete_clicked), 
NULL);
  
return toolbutton;

}



GtkWidget * 
xhtmlite_toolbutton_clear_formatting () 
{
  
GtkWidget * toolbutton =
    (GtkWidget *) gtk_tool_button_new_from_stock ("gtk-remove");
  
gtk_widget_show (toolbutton);
  
GtkTooltips * tooltips = gtk_tooltips_new ();
  
gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
			      _("Clear Formatting"), NULL);
  

g_signal_connect ((gpointer) toolbutton, "clicked",
		      
G_CALLBACK
		      (on_toolbutton_clear_formatting_clicked), 
NULL);
  
return toolbutton;

}



GtkWidget * 
xhtmlite_toolbutton_undo () 
{
	GtkWidget * toolbutton = (GtkWidget *) gtk_tool_button_new_from_stock ("gtk-undo");
	gtk_widget_show (toolbutton);
	GtkTooltips * tooltips = gtk_tooltips_new ();
	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
			_("Undo"), NULL);

	g_signal_connect ((gpointer) toolbutton, "clicked",
			G_CALLBACK (on_toolbutton_undo_clicked), NULL);
  
	return toolbutton;
}



GtkWidget * 
xhtmlite_toolbutton_redo () 
{
	GtkWidget * toolbutton = (GtkWidget *) gtk_tool_button_new_from_stock ("gtk-redo");
	gtk_widget_show (toolbutton);
	GtkTooltips * tooltips = gtk_tooltips_new ();
	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
			_("Redo"), NULL);

	g_signal_connect ((gpointer) toolbutton, "clicked",  
			G_CALLBACK (on_toolbutton_redo_clicked), NULL);

	return toolbutton;
}



GtkWidget * 
xhtmlite_toggletoolbutton_bold () 
{
  
GtkWidget * toggletoolbutton = (GtkWidget *) gtk_toggle_tool_button_new ();
  
gtk_tool_button_set_label (GTK_TOOL_BUTTON (toggletoolbutton), "Bold");
  
GtkWidget * tmp_image = create_pixmap (toggletoolbutton, "text_bold.png");
  
gtk_widget_show (tmp_image);
  
gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (toggletoolbutton),
				    tmp_image);
  
gtk_widget_show (toggletoolbutton);
  
GtkTooltips * tooltips = gtk_tooltips_new ();
  
gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toggletoolbutton),
			      tooltips, _("Bold"), NULL);
  

g_signal_connect ((gpointer) toggletoolbutton, "toggled",
		      
G_CALLBACK (on_toggletoolbutton_bold_toggled), 
NULL);
  
return toggletoolbutton;

}



GtkWidget * 
xhtmlite_toggletoolbutton_italic () 
{
  
GtkWidget * toggletoolbutton = (GtkWidget *) gtk_toggle_tool_button_new ();
  
gtk_tool_button_set_label (GTK_TOOL_BUTTON (toggletoolbutton), "Italic");
  
GtkWidget * tmp_image =
    create_pixmap (toggletoolbutton, "text_italic.png");
  
gtk_widget_show (tmp_image);
  
gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (toggletoolbutton),
				    tmp_image);
  
gtk_widget_show (toggletoolbutton);
  
GtkTooltips * tooltips = gtk_tooltips_new ();
  
gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toggletoolbutton),
			      tooltips, _("Italic"), NULL);
  

g_signal_connect ((gpointer) toggletoolbutton, "toggled",
		      
G_CALLBACK (on_toggletoolbutton_italic_toggled),
		      
NULL);
  
return toggletoolbutton;

}



GtkWidget * 
xhtmlite_toggletoolbutton_underline () 
{
  
GtkWidget * toggletoolbutton = (GtkWidget *) gtk_toggle_tool_button_new ();
  
gtk_tool_button_set_label (GTK_TOOL_BUTTON (toggletoolbutton),
			      "Underline");
  
GtkWidget * tmp_image = create_pixmap (toggletoolbutton, "text_under.png");
  
gtk_widget_show (tmp_image);
  
gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (toggletoolbutton),
				    tmp_image);
  
gtk_widget_show (toggletoolbutton);
  
GtkTooltips * tooltips = gtk_tooltips_new ();
  
gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toggletoolbutton),
			      tooltips, _("Underline"), NULL);
  

g_signal_connect ((gpointer) toggletoolbutton, "toggled",
		      
G_CALLBACK
		      (on_toggletoolbutton_underline_toggled), 
NULL);
  
return toggletoolbutton;

}

GtkWidget * 
xhtmlite_toggletoolbutton_strike () 
{
	GtkWidget * toggletoolbutton = (GtkWidget *) gtk_toggle_tool_button_new ();
	gtk_tool_button_set_label (GTK_TOOL_BUTTON (toggletoolbutton), "Strike");
	GtkWidget * tmp_image =
		create_pixmap (toggletoolbutton, "text_strike.png");
	gtk_widget_show (tmp_image);

	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (toggletoolbutton),
	    tmp_image);
	gtk_widget_show (toggletoolbutton);

	GtkTooltips * tooltips = gtk_tooltips_new ();
	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toggletoolbutton),
	    tooltips, _("Strike"), NULL);

	g_signal_connect ((gpointer) toggletoolbutton, "toggled",
		G_CALLBACK (on_toggletoolbutton_strike_toggled), NULL);
	return toggletoolbutton;
}


GtkWidget * 
xhtmlite_toggletoolbutton_superscript ()
{
	GtkWidget * toggletoolbutton = (GtkWidget *) gtk_toggle_tool_button_new ();
	gtk_tool_button_set_label (GTK_TOOL_BUTTON (toggletoolbutton), "Superscript");
	GtkWidget * tmp_image = create_pixmap (toggletoolbutton, "text_super.png");
	gtk_widget_show (tmp_image);
	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (toggletoolbutton), tmp_image);
	gtk_widget_show (toggletoolbutton);
	GtkTooltips * tooltips = gtk_tooltips_new ();
  
	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toggletoolbutton),
			tooltips, _("Superscript"), NULL);
	g_signal_connect ((gpointer) toggletoolbutton, "toggled", 
			G_CALLBACK (on_toggletoolbutton_superscript_toggled), NULL);
	
	return toggletoolbutton;
}

GtkWidget *
xhtmlite_toggletoolbutton_subscript () 
{
	GtkWidget * toggletoolbutton = (GtkWidget *) gtk_toggle_tool_button_new ();
	gtk_tool_button_set_label (GTK_TOOL_BUTTON (toggletoolbutton),
			      "Subscript");

	GtkWidget * tmp_image = create_pixmap (toggletoolbutton, "text_sub.png");
	gtk_widget_show (tmp_image);
	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (toggletoolbutton), tmp_image);
	gtk_widget_show (toggletoolbutton);

	GtkTooltips * tooltips = gtk_tooltips_new ();
	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toggletoolbutton),
			      tooltips, _("Subscript"), NULL);
	g_signal_connect ((gpointer) toggletoolbutton, "toggled", G_CALLBACK
		      (on_toggletoolbutton_subscript_toggled), NULL);

	return toggletoolbutton;
}



GtkWidget * 
		xhtmlite_toolbutton_symbol (GtkWidget * parent_window)
{
	GtkWidget * tmp_image = create_pixmap (parent_window, "sum.png");
	gtk_widget_show (tmp_image);

	GtkWidget * toolbutton =
			(GtkWidget *) gtk_tool_button_new (tmp_image, _("Add Special Characters"));
	gtk_widget_show (toolbutton);

	GtkTooltips * tooltips = gtk_tooltips_new ();
	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
				   _("Add Special Characters"), NULL);

	g_signal_connect ((gpointer) toolbutton, "clicked",
			   G_CALLBACK (on_toolbutton_symbol_clicked),
			   NULL);
	return toolbutton;
}



GtkWidget * 
		xhtmlite_toolbutton_inline_image (GtkWidget * parent_window)
{
	GtkWidget * tmp_image = create_pixmap (parent_window, "inline_image.png");
	gtk_widget_show (tmp_image);
	GtkWidget * toolbutton =
		(GtkWidget *) gtk_tool_button_new (tmp_image, _("Image"));
	gtk_widget_show (toolbutton);
  

	GtkTooltips * tooltips = gtk_tooltips_new ();
	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
		_("Import Image to Inline"), NULL);
  
	g_signal_connect ((gpointer) toolbutton, "clicked",
		G_CALLBACK (on_toolbutton_inline_image_clicked),
		 NULL);
	return toolbutton;
}



GtkWidget * 
xhtmlite_combobox_font (GtkWidget * parent_window)
{
	GtkWidget * toolitem = (GtkWidget *) gtk_tool_item_new ();
	gtk_widget_show (toolitem);
	GtkListStore * store = gtk_list_store_new (1, G_TYPE_STRING);
	GtkWidget * combobox = gtk_combo_box_entry_new_with_model (GTK_TREE_MODEL (store), 0);
	gtk_combo_box_set_focus_on_click (GTK_COMBO_BOX (combobox), FALSE);
	
	GLADE_HOOKUP_OBJECT (parent_window, combobox, "combobox_font");
  
	gtk_widget_show (combobox);
	gtk_container_add (GTK_CONTAINER (toolitem), combobox);
	gtk_container_set_border_width (GTK_CONTAINER (combobox), 4);

	// Get default fonts
	GtkWidget * textview = gtk_text_view_new ();

	PangoContext * textview_context =
			gtk_widget_get_pango_context (GTK_WIDGET (textview));

	PangoFontFamily ** families;
	int len, n, default_fontfamily_listn;
	pango_context_list_families (textview_context, &families, &len);
	qsort (families, len, sizeof (PangoFontFamily *), cmp_families);
  
	PangoFontDescription * default_font_desc =
			gtk_widget_get_style (GTK_WIDGET (textview))->font_desc;
  
	const gchar *default_fontfamily =
			pango_font_description_get_family (default_font_desc);
  
	gint default_fontsize =
			pango_font_description_get_size (default_font_desc);

	// setup combobox_font with all the font families
	GtkTreeIter iter;
	for (n = 0; n < len; n++)
	{
		const gchar *fontname = pango_font_family_get_name (families[n]);
		// gtk_combo_box_append_text(GTK_COMBO_BOX(combobox_font), fontname);
		gtk_list_store_append (store, &iter);
		gtk_list_store_set (store, &iter, 0, fontname, -1);
      
		if (g_ascii_strcasecmp (default_fontfamily, fontname) == 0)
		{
			default_fontfamily_listn = n;
		}
	}
	gtk_combo_box_set_active (GTK_COMBO_BOX (combobox),
				  default_fontfamily_listn);
	g_free (families);
	gtk_widget_destroy (textview);
	g_signal_connect ((gpointer) combobox, "changed",
			   G_CALLBACK (on_combobox_font_changed), NULL);
  
	return toolitem;
}



GtkWidget * 
		xhtmlite_combobox_fontsize_absolute (GtkWidget *
		parent_window)
{
	GtkWidget * toolitem = (GtkWidget *) gtk_tool_item_new ();
	gtk_widget_show (toolitem);

	GtkListStore * store = gtk_list_store_new (1, G_TYPE_STRING);
	GtkWidget * combobox = gtk_combo_box_entry_new_with_model (GTK_TREE_MODEL (store), 0);
	gtk_combo_box_set_focus_on_click (GTK_COMBO_BOX (combobox), FALSE);

	GLADE_HOOKUP_OBJECT (parent_window, combobox, "combobox_fontsize_absolute");
	
	gtk_widget_show (combobox);
	gtk_container_add (GTK_CONTAINER (toolitem), combobox);
	gtk_container_set_border_width (GTK_CONTAINER (combobox), 4);
	
	// setup combobox_fontsize_absolute
	GtkTreeIter iter;
	gchar * sizes[8] =
	{"Auto-Size", "xx-small", "x-small", "small", "medium",
		"large", "x-large", "xx-large"};

		int n;
		for (n = 0; n < 8; n++)
		{
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter, 0, g_strdup (sizes[n]), -1);
		}
		gtk_combo_box_set_active (GTK_COMBO_BOX (combobox), 0);
		g_signal_connect ((gpointer) combobox, "changed",
				   G_CALLBACK(on_combobox_fontsize_absolute_changed), NULL);

		return toolitem;
}


GtkWidget * 
		xhtmlite_combobox_fontsize_pt (GtkWidget * parent_window)
{
	GtkWidget * toolitem = (GtkWidget *) gtk_tool_item_new ();
	gtk_widget_show (toolitem);
  
	GtkWidget * combobox = gtk_combo_box_entry_new_text ();
	gtk_combo_box_set_focus_on_click (GTK_COMBO_BOX (combobox), FALSE);
	gtk_widget_set_size_request (combobox, 50, -1);
	GLADE_HOOKUP_OBJECT (parent_window, combobox, "combobox_fontsize_pt");
	
	// gtk_container_set_border_width (GTK_CONTAINER (combobox), 4);
	gtk_widget_show (combobox);
	gtk_container_add (GTK_CONTAINER (toolitem), combobox);
	gtk_container_set_border_width (GTK_CONTAINER (combobox), 4);
  
	g_signal_connect ((gpointer) combobox, "changed",
			   G_CALLBACK (on_combobox_fontsize_absolute_changed), NULL);
	
	return toolitem;
}


GtkWidget * 
		xhtmlite_toolbutton_fontcolor ()
{
	GtkWidget *toolbutton = (GtkWidget *) gtk_tool_button_new_from_stock ("gtk-select-color");
	gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolbutton), _("Font Color"));
	GtkWidget *tmp_image = create_pixmap (toolbutton, "colorize.png");
	gtk_widget_show (tmp_image);
	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (toolbutton), tmp_image);
  
	gtk_widget_show (toolbutton);
	GtkTooltips *tooltips = gtk_tooltips_new ();
	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
				   _("Select Font Color"), NULL);
  
	g_signal_connect ((gpointer) toolbutton, "clicked",
			   G_CALLBACK (on_toolbutton_fontcolor_clicked), NULL);
  
	return toolbutton;
}

GtkWidget * 
xhtmlite_toolbutton_highlightcolor ()
{
	GtkWidget * tmp_image =
			gtk_image_new_from_stock ("gtk-select-color", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (tmp_image);
	GtkWidget * toolbutton = (GtkWidget *) gtk_tool_button_new (tmp_image, _("Highlight Color"));
  
	gtk_widget_show (toolbutton);
	GtkTooltips * tooltips = gtk_tooltips_new ();
	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
				   _("Select Highlight Color"), NULL);
  
	g_signal_connect ((gpointer) toolbutton, "clicked",
			   G_CALLBACK (on_toolbutton_highlightcolor_clicked), NULL);
	
	return toolbutton;
}

GtkWidget* 
xhtmlite_toolbutton_insert_link()
{
	GtkWidget *toolbutton = (GtkWidget *) gtk_tool_button_new_from_stock ("gtk-network");
	gtk_tool_button_set_label (GTK_TOOL_BUTTON (toolbutton), _("Link"));
	GtkWidget *tmp_image = create_pixmap (toolbutton, "link.png");
	gtk_widget_show (tmp_image);
	gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (toolbutton), tmp_image);

	gtk_widget_show (toolbutton);
	GtkTooltips * tooltips = gtk_tooltips_new ();
	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (toolbutton), tooltips,
				   _("Insert Link"), NULL);
  
	g_signal_connect ((gpointer) toolbutton, "clicked",
			   G_CALLBACK (on_toolbutton_insert_link_clicked), NULL);
	
	return toolbutton;
}
