<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xs:template-name>Spartan</xs:template-name>
<xs:template-description>A basic BYOJeopardy html template with no graphics or css styles.  This produces html files that take up very little space and run very fast.  Requires Javascript enabled on the browser.</xs:template-description>
<xs:template-image></xs:template-image>

<xsl:output method="html"/>
<xsl:template match="byojeopardy">
<HTML>
<HEAD>
<TITLE><xsl:text>BYOJeopardy -- </xsl:text><xsl:value-of select="$OutputFileName"/></TITLE>

  <SCRIPT type="text/javascript">
<![CDATA[
function Show_Question()
{
	document.getElementById('board').style.display = "none";
	document.getElementById('question').style.display = "";
}
function Hide_Question(BD, QD)
{
	document.getElementById('board').style.display = "";
	document.getElementById('question').style.display = "none";
}
function Show_Answer()
{
	if (document.getElementById('question_link').innerHTML == 'Answer')
	{
		document.getElementById('question_display').innerHTML = document.getElementById('question_display').ans_string;
		document.getElementById('question_link').innerHTML = 'Question';
	}
	else
	{
		document.getElementById('question_display').innerHTML= document.getElementById('question_display').ques_string;
		document.getElementById('question_link').innerHTML = 'Answer';
	}
}
function Disable_Link(BtnLink,QuesString,AnsString)
{
 document.getElementById(BtnLink).onclick=new Function('javascript: return false;');
 document.getElementById(BtnLink).style.color='rgb(128, 147, 176)';
 document.getElementById('question_display').ques_string = QuesString;
 document.getElementById('question_display').ans_string = AnsString;
]]>
     <xsl:choose>
        <xsl:when test="$answer_in_form_of_question = 1">
                  <![CDATA[
	              document.getElementById('question_display').innerHTML = AnsString;
	              document.getElementById('question_link').innerHTML = 'Question';
                  ]]>
        </xsl:when>
        <xsl:otherwise>
                  <![CDATA[
	              document.getElementById('question_display').innerHTML = QuesString;
	              document.getElementById('question_link').innerHTML = 'Answer';
                  ]]>
        </xsl:otherwise>
     </xsl:choose>
     <![CDATA[ 
     return true;
     }]]>
</SCRIPT>

</HEAD>

<BODY>
<SPAN id="board" style="">
<TABLE align="center" cellspacing="5">
    <TR>
       <xsl:apply-templates select="topic"/>
    </TR>
		<TR height="5" />
    <xsl:call-template name="rowCounter">
      <xsl:with-param name="N" select="1"/>
   </xsl:call-template>
  </TABLE>
</SPAN>

<span id="question" style="display: none;">
<table align="center" cellspacing="5">
		<tbody>
		<tr>
		<th colspan="2"> <span id="question_display"
		ques_string="" ans_string=""></span></th>
		</tr>
		<tr>
		<td> <a href="javascript:Hide_Question()">Back
		to Board</a> </td>
		<td> <a id="question_link"
		href="javascript:Show_Answer()">Answer</a>
		</td>
		</tr>
		</tbody>
		</table>
		</span>
		<br />
		<xsl:comment>
		We request you retain the full copyright notice below including the link to www.wolfescience.com.
		This not only gives respect to the large amount of time given freely by the developers
		but also helps build interest, traffic and use of www.wolfescience.com. If you cannot (for good
		reason) retain the full copyright we request you at least leave in place the
		Created by BYOJeopardy at available from www.wolfescience.com line, with wolfescience linked to www.wolfescience.com. If you refuse
		to include even this then support on our forums may be affected.
		Doug Wolfe: 2006
		</xsl:comment>
		<div align="center"><font size="-2">created
		by BYOJeopardy <br />
		available from <a href="http://www.wolfescience.com/">www.wolfescience.com</a>
		</font></div>

     </BODY>
</HTML>
</xsl:template>

<xsl:template name="rowCounter">
   <xsl:param name="N" /><!--Row Counter-->
   <!-- If there are any values in any section -->

<xsl:if test="topic/value[ $N ]">
		
     <TR>
      <!-- We produce a TD for each section, -->
	<xsl:for-each select="topic">
		<xsl:if test="topic_name != 'Daily Double'">
		
		<TD>

		<xsl:variable name="question_variable">
			<xsl:call-template name="fixquotes">
			<xsl:with-param name="string" select="value[ $N ]/question"/>
			</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="answer_variable">
			<xsl:call-template name="fixquotes">
			<xsl:with-param name="string" select="value[ $N ]/answer"/>
		</xsl:call-template>
		</xsl:variable>
		<xsl:variable name="link_id" select="concat(position(), $N)"/>
		<a href="javascript:Show_Question()" id="{$link_id}" onclick="javascript:Disable_Link('{$link_id}','{$question_variable}','{$answer_variable}');" color="">
		<xsl:apply-templates select="value[ $N ]/amount"/></a>
          	     	<!--xsl:value-of select="amount"/-->
			<!--xsl:value-of select="question"/-->
			<!--xsl:value-of select="answer"/-->
                  <!-- This space is needed to stop us
            getting empty TD elements which XT outputs as <TD/> which
	     Netscape doesn't like. -->

        </TD>
		</xsl:if>
      </xsl:for-each>

     </TR>
     <!-- The recursive call for the next larger
       value of N -->
<xsl:call-template name="rowCounter">
       <xsl:with-param name="N" select="$N + 1"/>
</xsl:call-template>

		</xsl:if>

</xsl:template>

<xsl:template match="topic">
		<xsl:if test="topic_name != 'Daily Double'">
		<TH><xsl:value-of select="topic_name"/></TH>
		</xsl:if>
</xsl:template>
			
<xsl:template name="fixquotes">
<!-- Escapes quotes -->
	<xsl:param name="string"/>
	<xsl:choose>
	<xsl:when test="contains($string, &quot;'&quot;)">
		<xsl:value-of select="substring-before($string, &quot;'&quot;)"/>
		<xsl:text>\'</xsl:text>
	<xsl:call-template name="fixquotes">
	<xsl:with-param name="string"
	select="substring-after($string, &quot;'&quot;)"/>
	</xsl:call-template>
	</xsl:when>
	<xsl:otherwise>
	<xsl:value-of select="$string"/>
	</xsl:otherwise>
	</xsl:choose>
</xsl:template>

</xsl:stylesheet>
