/*
 * xhtmlcallbacks.c
 * xhtmlite
 *
 * by Doug Wolfe
 * doug@wolfescience.com 
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>

#include "support.h"
#include "base64.h"

#include "xhtmlcallbacks.h"
#include "xhtmlinterface.h"
#include "xhtmlbuffer.h"
#include "xhtmlmenu.h"
#include "xhtmlundo.h"

GtkTextBuffer *buffer;
GtkTextView *textview;
GtkTextBuffer *buffer_output;

void xhtmlite_dialog_setup_buffer (GtkWidget *widget, gchar *text, gboolean *debug)
{
     //create textview buffer
	textview = GTK_TEXT_VIEW(lookup_widget(widget, "xhtml_dialog_textview"));
	gtk_widget_set_name(GTK_WIDGET(textview), "xhtmlite_textview");
	buffer = xhtmlite_create_buffer (textview, TRUE);
	GtkWidget *statusbar = lookup_widget(widget, "xhtmlite_dialog_statusbar");
	xhtmlite_attach_textview_events(GTK_WIDGET(textview), statusbar);
	
	//create textview_output buffer
	GtkWidget *textview_output = lookup_widget(widget, "xhtml_dialog_textview_output");
	GtkTextTagTable *tagtable_output =  gtk_text_tag_table_new();
	buffer_output = gtk_text_buffer_new (tagtable_output);
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(textview_output) , buffer_output);
	
	//add the toolbars
	GtkWidget *dialog_vbox = lookup_widget(widget, "xhtml_dialog_vbox");
	GtkWidget *toolbar = xhtmlite_create_default_toolbar_edit(widget);
    gtk_box_pack_start (GTK_BOX (dialog_vbox), toolbar, FALSE, TRUE, 0);
   	gtk_box_reorder_child (GTK_BOX (dialog_vbox),toolbar,0);
	toolbar = xhtmlite_create_default_toolbar_font(widget);
	GtkWidget *toolitem_combobox_fontsize_pt = lookup_widget(widget, "toolitem_combobox_fontsize_pt");
	gtk_widget_show_all(toolbar);
    gtk_box_pack_start (GTK_BOX (dialog_vbox), toolbar, FALSE, TRUE, 0);
	gtk_box_reorder_child (GTK_BOX (dialog_vbox),toolbar,4);
	gtk_widget_show_all(toolbar);
	gtk_widget_hide(toolitem_combobox_fontsize_pt);
	
	//add the undo/redo
	GtkWidget *undo_button = lookup_widget(widget, "toolbutton_undo");
	GtkWidget *redo_button = lookup_widget(widget, "toolbutton_redo");
	xhtmlite_undo_assign_buttons(buffer, undo_button, redo_button);
	
	//toggle display of textview_Output and Debugging buttons (Export/Import XHTML buttons)
    if(!debug)
	{
		GtkWidget *vbox_debug = lookup_widget(GTK_WIDGET(widget), "vbox_debug");
		gtk_widget_hide(vbox_debug);
	}
	xhtmlite_set(buffer, text);
}
 

void xhtml_dialog_button_export_on_clicked (GtkButton *button, gpointer user_data)
{
	GtkTextIter begin, end;
	gtk_text_buffer_get_bounds (buffer, &begin, &end);

	//gchar *text =  gtk_text_buffer_get_text(buffer, &begin, &end, TRUE);
	gchar *xhtml = xhtmlite_write(buffer);
	gtk_text_buffer_set_text (buffer_output, xhtml, -1);
}

void xhtml_dialog_button_import_on_clicked (GtkButton *button, gpointer user_data)
{
	//Get XHTML from output textview
	GtkTextIter begin, end;
	gtk_text_buffer_get_bounds  (buffer_output, &begin, &end);

	gchar *xml =  gtk_text_buffer_get_text(buffer_output, &begin, &end, TRUE);
	printf("XHTML:\n%s\n", xml);
	xhtmlite_import (buffer, xml, FALSE);
}

gchar *xhtml_response = NULL;

void xhtml_dialog_on_response (GtkDialog *dialog, gint response_id, gpointer user_data)
{
	if (response_id == GTK_RESPONSE_OK)
	{
		xhtml_response = xhtmlite_write(buffer);
		printf("return XHTML:%s", xhtml_response);
	}
	gtk_widget_hide(GTK_WIDGET(dialog));
}

void xhtmlite_symbol_combobox_unicodetype_on_changed (GtkComboBox * combobox, gpointer user_data)
{
	GtkWidget *xhtml_symbol_table = lookup_widget(GTK_WIDGET(combobox), "xhtmlite_symbol_table");
	//rebuild table
	gtk_container_foreach (GTK_CONTAINER (xhtml_symbol_table),
			       (GtkCallback) gtk_widget_destroy,
			       NULL);
	
	int n, r, c;
	r=0;
	c=0;

	int start_char = 0;
	int end_char = 0;
	
	int chosen_unicode_type = gtk_combo_box_get_active (GTK_COMBO_BOX(combobox));
	//char *chosen_unicode_text=  gtk_combo_box_get_active_text(GTK_COMBO_BOX(combobox));
	printf("Chosen Unicode Type:%d", chosen_unicode_type);
	switch (chosen_unicode_type)
	{
		case (0): //Arabic  U+0600  -   U+06FF  (1536-1791)
			start_char = 1536;
			end_char = 1791;
			break;
		case (1): //Arrows Unicode U+2190 ? U+21FF   (8592-8703)
			start_char = 8592;
			end_char = 8703;
			break;
//		case (1): //Arrows-A Supplemental U+2190 ? U+21FF   (8592-8703)
//			start_char = 8592;
//			end_char = 8703;
//			break;
//		case (1): //Arrows-B Supplemental U+2190 ? U+21FF   (8592-8703)
//			start_char = 8592;
//			end_char = 8703;
//			break;
		case (2): //Basic Latin Unicode U+0000 ? U+007F   (0-127)
			start_char = 0;
			end_char = 127;
			break;
		case (3): //Currency Symbols Unicode U+20A0 ? U+20CF   (8352-8399)
			start_char = 0;
			end_char = 127;
			break;
		case (4): //General Punctuation Unicode U+2000 ? U+206F   (8192-8303)
			start_char = 8192;
			end_char = 8303;
			break;
		case (5): //Geometric Shapes Unicode U+25A0 ? U+25FF   (9632-9727)
			start_char = 9632;
			end_char = 9727;
			break;
		case (6): //Greek Unicode U+0370 ? U+03FF   (880-1023)
			start_char = 880;
			end_char = 1023;
			break;
		case (7): //Latin-1 Supplement Unicode U+0080 ? U+00FF   (128-255)
			start_char = 128;
			end_char = 255;
			break;
		case (8): //Latin Extended-A Unicode U+0100 ? U+017F   (256-383)
			start_char = 256;
			end_char = 383;
			break;
		case (9): //Latin Extended-B Unicode U+0180 ? U+024F   (384-591)
			start_char = 384;
			end_char = 591;
			break;
		case (10): //Letterlike Symbols Unicode U+2100 ? U+214F   (8448-8527)
			start_char = 8448;
			end_char = 8527;
			break;
		case (11): //Mathematical Operators Unicode U+2200 ? U+22FF   (8704-8959)
			start_char = 8704;
			end_char = 8959;
			break;
		case (12): //Miscellaneous Technical Unicode U+2300 ? U+23FF   (8960-9215)
			start_char = 8960;
			end_char = 9215;
			break;
		case (13): //Miscellaneous Symbols Unicode U+2600 ? U+26FF   (9728-9983)
			start_char = 9728;
			end_char = 9983;
			break;
		case (14): //Spacing Modifier Letters Unicode U+02B0 ? U+02FF   (688-767)
			start_char = 688;
			end_char = 767;
			break;
		default:
			printf("other");
	}
	for(n=start_char; n<end_char; n++)
	{
		static gchar buf[10];
		int len;
		len = g_unichar_to_utf8 (n, buf);
		buf[len] = '\0';

		//printf("n:%d, buf:%s\n", n, buf);
		const gchar *end;
		if(g_utf8_validate (buf,  -1, &end) && !g_unichar_isspace (buf[0]))
		{
			if (strcmp(buf, ""))
			{
				GtkWidget *btnChar;
				btnChar = gtk_button_new_with_label (buf);
				gtk_button_set_relief(GTK_BUTTON(btnChar), GTK_RELIEF_NONE);
				gtk_table_attach_defaults   (GTK_TABLE(xhtml_symbol_table),
						btnChar,
						c,
						c+1,
						r,
						r+1);
				g_signal_connect ((gpointer) btnChar, "enter",
						G_CALLBACK (xhtmlite_on_button_symbol_activate),
						NULL);
				g_signal_connect ((gpointer) btnChar, "clicked",
						G_CALLBACK (xhtmlite_on_button_symbol_clicked),
						NULL);
				c += 1;
				if (c > 10)
				{
					c = 0;
					r += 1;
				}
				gtk_widget_show(btnChar);
			}
		}
		else
		{
			//g_print(" invalid\n");
		}
	}
}
